/* Libraries part */
#include <Wire.h>
#include <SparkFun_VL53L5CX_Library.h>
#include "NanoEdgeAI.h"
#include "knowledge.h"

/* Macros definitions */
#define SERIAL_BAUD_RATE          115200

/* Frame resolution: 4x4 or 8x8 */
#define SENSOR_FRAME_RESOLUTION  64

/* Number of successive frames to be collected */
#define SENSOR_FRAMES 1

/* Sensor data rate */
#define SENSOR_DATA_RATE  15

/* NanoEdgeAI defines part
   NEAI_MODE = 1: NanoEdgeAI functions = AI Mode.
   NEAI_MODE = 0: Datalogging mode.
*/

/* Depending on your target, you need to adapt
   the i2c connection. For example, you have to use
   "Wire" with Arduino Uno R4 Wifi but, "Wire1" with
   the Arduino Giga R1 board. Check your hardware !
*/
#define i2c_comm Wire

/* In this example, we use I2C connection */
SparkFun_VL53L5CX myImager;
VL53L5CX_ResultsData measurementData;

/* variable to play rock paper scissors */
static uint16_t player_score = 0;
static uint16_t arduino_score = 0;
static uint16_t points_to_win = 3;


/* Global variables definitions */
static uint16_t neai_ptr = 0;
static float neai_buffer[SENSOR_FRAME_RESOLUTION * SENSOR_FRAMES] = {0.0};

/* NEAI library variables */
static uint8_t neai_code = 0;
static uint16_t id_class = 0;
static float output_class_buffer[CLASS_NUMBER];
const char *id2class[CLASS_NUMBER + 1] = { // Buffer for mapping class id to class name
  "unknown",
  "nothing",
  "scissors",
  "rock",
  "paper",
};

void fill_buffer();

/* Initialization function: In this function,
    code runs only once at boot / reset.
*/
void setup() {
  /* Init serial at baud rate 115200 */
  Serial.begin(SERIAL_BAUD_RATE);

  /* I2C workaround: Sometimes, on some boards,
     I2C get stuck after software reboot, reset so,
     to avoid this, we toggle I2C clock pin at boot.
  */
  pinMode(SCL, OUTPUT);
  for (uint8_t i = 0; i < 20; i++) {
    digitalWrite(SCL, !digitalRead(SCL));
    delay(1);
  }
  delay(100);

  /* Init I2C connection between board & sensor */
  i2c_comm.begin();
  i2c_comm.setClock(400000);
  if (myImager.begin() == false) {
    Serial.print("Sensor not found - check your wiring. Freezing.\n");
    while (1);
  }

  /* Init VL53L5CX sensor settings: resolution & data rate */
  myImager.setResolution(SENSOR_FRAME_RESOLUTION);
  myImager.setRangingFrequency(SENSOR_DATA_RATE);
  myImager.startRanging();

  /* Initialize NanoEdgeAI AI */
  neai_code = neai_classification_init(knowledge);
  if (neai_code != NEAI_OK) {
    Serial.print("Not supported board.\n");
  }
}

/* Main function: Code run indefinitely */
void loop() {

  Serial.println("The game is starting, be ready");
  delay(1000);

  while ((arduino_score < points_to_win) and (player_score < points_to_win)) {
    Serial.println();
    Serial.print("Scissor");
    delay(1000);
    Serial.print(" Paper");
    delay(1000);
    Serial.print(" Rock!!");
    Serial.println();
    delay(1000);


    fill_buffer();
    //classification
    neai_classification(neai_buffer, output_class_buffer, &id_class);
    
    while (id_class == 1) {
      //collect tof data for classification
      fill_buffer();
      //classification
      neai_classification(neai_buffer, output_class_buffer, &id_class);
      Serial.println("You didn't make a sign, play again");
      delay(1000);

    }


    //pick a sign played by the board
    int random_play = random(2, 5);


    Serial.print("you played: ");
    Serial.println(id2class[id_class]);
    Serial.print("Arduino played: ");
    Serial.println(id2class[random_play]);
    delay(100);

    //comparison
    if (id_class == random_play) {
      Serial.println("Draw, play again!");
    } else {
      if (
        // scisoor vs paper
        (id_class == 2 && random_play == 4) or
        // rock vs scissors
        (id_class == 3 && random_play == 2) or
        //paper vs rock
        (id_class == 4 && random_play == 3)
      ) {
        Serial.println("You won");
        player_score ++;
      } else {
        Serial.println("You loose");
        arduino_score ++;
      }
      Serial.print("Player Score: ");
      Serial.println(player_score);
      Serial.print("Arduino Score: ");
      Serial.println(arduino_score);
      delay(1000);
    }

    /* Clean neai buffer */
    memset(neai_buffer, 0.0, (uint16_t) (SENSOR_FRAME_RESOLUTION * SENSOR_FRAMES) * sizeof(float));
  }
  Serial.println(" ");
  if (player_score > arduino_score) {
    Serial.println("RESULTS: Well done, you won");
  } else {
    Serial.println("RESULTS:Too bad, you lost");
  }
  Serial.println(" ");

  arduino_score = 0;
  player_score = 0;
  delay(1000);
}


void fill_buffer() {
  while (neai_ptr < SENSOR_FRAMES) {
    /* Check if new data if available */
    if (myImager.isDataReady() == true) {
      /* If new data is available we read it ! */
      myImager.getRangingData(&measurementData);
      /* Get data in the neai buffer */
      for (uint16_t i = 0; i < SENSOR_FRAME_RESOLUTION; i++) {
        neai_buffer[i + (SENSOR_FRAME_RESOLUTION * neai_ptr)] = (float) measurementData.distance_mm[i];
      }
      neai_ptr++;
    }
  }
  /* Reset pointer */
  neai_ptr = 0;
}
